'' +--------------------------------------------------------------------------+
'' | Cluso's RamBlade & TriBladeProp:  Determine Hardware               v1.00 |
'' +--------------------------------------------------------------------------+
'' |  Author:            "Cluso99" (Ray Rodrick)                              |
'' |  Copyright (c) 2012 "Cluso99" (Ray Rodrick)                              |
'' |  License            MIT License - See end of file for terms of use       |
'' +--------------------------------------------------------------------------+
''
'' This object will determine the hardware we are running on
'' Checks for:
''      TriBlade#2
''      RamBlade
''      RamBlade3
''
'' RR20120303   as used in PropBootSD_V2_00  combo eeprom boot program
''


CON


'Now common for TriBlade#2, RamBlade & RamBlade3
  _clkmode  = xtal1 + pll16x
  _xinfreq  = 6_500_000         'currently only 6.5MHz
  
  SIpin     = 31                'serial                 '23 \RamBlade
  SOpin     = 30                                        '22 /
' SIOmode   = 0
' baud      = 115200
  tb2DO     =  9                '\ TriBlade2 microSD
  tb2Clk    = 28                '|
  tb2DI     =  8                '|
  tb2CS     = 14                '|
  tb2PU     = $3000_FF00        '/ pullups
  rb1DO     = 24                '\ RamBlade  microSD (aka RamBlade1)
  rb1Clk    = 26                '|
  rb1DI     = 25                '|
  rb1CS     = 19                '|
  rb1PU     = $3738_0000        '/ pullups
  rb3DO     = 20                '\ RamBlade3 microSD
  rb3Clk    = 21                '|
  rb3DI     = 22                '|
  rb3CS     = 23                '|
  rb3PU     = $3880_0000        '/ pullups

  'hardware value
  rb1val    = 1                 'RamBlade 
  tb2val    = 2                 'TriBlade#2
  rb3val    = 3                 'RamBlade3
  rb1 = rb1val << 29 | rb1CS << 24 | rb1DI << 16 | rb1Clk << 8 | tb2DO
  tb2 = tb2val << 29 | tb2CS << 24 | tb2DI << 16 | tb2Clk << 8 | tb2DO
  rb3 = rb3val << 29 | rb3CS << 24 | rb3DI << 16 | rb3Clk << 8 | rb3DO

     
PUB DetectHardware : HwVal | pu
'determine what hardware we are running on
  pu := ina                                             'read the input pins

  if     (pu & tb2PU) == tb2PU                          'TriBlade#2 ?
    Return tb2
  elseif (pu & rb1PU) == rb1PU                          'RamBlade ?
    Return rb1
  elseif (pu & rb3PU) == rb3PU                          'RamBlade3 ?
    Return rb3
  else                                                  'hardware invalid  
    Return -1                                           


DAT                                                   
{{
+------------------------------------------------------------------------------------------------------------------------------+
|                                                   TERMS OF USE: MIT License                                                  |                                                            
+------------------------------------------------------------------------------------------------------------------------------+
|Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation    | 
|files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,    |
|modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software|
|is furnished to do so, subject to the following conditions:                                                                   |
|                                                                                                                              |
|The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.|
|                                                                                                                              |
|THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE          |
|WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR         |
|COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,   |
|ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                         |
+------------------------------------------------------------------------------------------------------------------------------+
}} 