{{
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Initial Dos Boot module - calls _VER.CMD 
//
// Derived from SD3.01 FATEngine Demo
// Author:        1/2/2011      Kwabena W. Agyeman     
// Modifications: J. Moxham Dr_Acula to create KyeDOS v3.
// Modifications: Cluso99 (Ray Rodrick) break into modules
// Copyright:     (c) 2011 Kwabena W. Agyeman  (Kye)
//                (c) 2011 J. Moxham           (Dr_Acula)
//                (c) 2012 Ray Rodrick         (Cluso99)
// Licence:       MIT (see end of file for details)
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Modifications by Cluso99...
// RR20120306   000   Initial module _DOS.CMD 
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
}}



CON
  _clkmode = _hw#_clkmode
  _xinfreq = _hw#_xinfreq

  _nullCharacter                = 0
  _homeCursorCharacter          = 1
  _backspaceCharacter           = 8
  _horizontalTabCharacter       = 9
  _lineFeedCharacter            = 10
  _carriageReturnCharacter      = 13
  _clearScreenCharacter         = 16
  _quoteCharacter               = 34
  _backspaceKeycode             = 200



OBJ
  _hw   : "_hwdef"              'hardware pin definitions etc
  _hub  : "_hubdef"             'fixed hub definitions ($7FFF... down)

  hwdet : "DetectHardware_001"

  fat   : "SD-MMC_FATEngine_002.spin"
  taf   : "SD-MMC_FATEngine_002.spin"
  str   : "ASCII0_STREngine.spin"
  sio   : "pcFullDuplexSerial2FC" 

VAR

   byte CommandLine[80] ' complete command line as typed in
   byte LineOfText1[80] ' general purpose string buffer
   byte LineOfText2[80] ' general purpose string buffer
   byte LineOfText3[80] ' general purpose string buffer

'    long hardware               'hw def & pins

PUB shell | z,z0,z1,z2,z3,z4, InputPins, OutputPins, hardware

  long[_hub#_toggleLF] := false                         ' turn off auto lf after cr

'detect the hardware
  hardware := hwdet.DetectHardware
  if hardware == -1
    HardwareFail  

  case hardware >> 29
    1 :    InputPins  := _hw#_siRB1                     'RamBlade
           OutputPins := _hw#_soRB1
    2 :    InputPins  := _hw#_siPin                     'TriBlade
           OutputPins := _hw#_soPin
    3 :    InputPins  := _hw#_siPin                     'RamBlade3
           OutputPins := _hw#_soPin
    other: InputPins  := _hw#_siPin                     '??? use default P31/P30
           OutputPins := _hw#_soPin

'==================
'set parameters in hub for other modules
  long[_hub#_sdPins]  := hardware & $1F1F1F1F           
  long[_hub#_sioPins] := _hw#_sioMode << 16 | OutputPins << 8 | InputPins
  long[_hub#_sioBaud] := _hw#_sioBaud                              
  long[_hub#_toggleLF]:= false
  long[_hub#_hardware]:= 1 << 16 | hardware >> 29  '$0001_000x = cluso's hardware 
  str.copy(string("ver"),_hub#_stringBuffer)           
'==================
  

  waitcnt(clkfreq/1000 * 500 + cnt)                     ' delay 500ms
    
  z  :=  long[_hub#_sioPins]
  z0 :=  z        & $1F
  z1 := (z >>  8) & $1F
  z2 := (z >> 16) & $1F
' z3 := (z >> 24) & $1F
  z4 :=  long[_hub#_sioBaud]
  sio.AddPort(0, z0, z1, -1, -1, 0, z2, z4)
  sio.start                                             ' start the com port

  z  :=  long[_hub#_sdPins]
  z0 :=  z        & $1F
  z1 := (z >>  8) & $1F
  z2 := (z >> 16) & $1F
  z3 := (z >> 24) & $1F
  ifnot( fat.FATEngineStart(z0, z1,  z2, z3, -1,     -1,     -1,       -1,       -1))
' ifnot( fat.FATEngineStart(DO, CLK, DI, CS, _SD_WP, _SD_CD, _RTC_DAT, _RTC_CLK, -1))
    reboot

  fat.mountPartition(0)                                 ' mount the sd card

  PrintChar(_clearScreenCharacter)                      ' clear screen
  PrintDosVer                                           ' program message
  PrintStringCR(string(" Type Help for command listing"))

  ifnot(fat.partitionMounted)
    printstring(string("Card Not Mounted "))   

  sio.rxflush(0)                                        ' in case try to send an xmodem transfer
  crlf

'=================
' return to the command module (go via "_VER.CMD")
  str.copy(string("ver"),_hub#_stringBuffer)           
  \fat.bootPartition(string("_VER.CMD"))
  'only here on error
  PrintStringCR(string("=== error ==="))
  waitcnt(clkfreq/1000 * 500 + cnt)                     ' delay 500ms
  reboot



PRI PrintDosVer
  PrintStringCR(string("*** KyeDOS SD card operating system v3.10 ***"))
  PrintStringCR(string(" by Kwabena W. Agyeman & J. Moxham & Cluso99 "))
  PrintString(string("  Hardware: "))
  case long[_hub#_hardware] >> 16  'hardware >> 29
    $0001 : PrintString(string("Cluso's "))
    other : PrintString(string("??? "))
  case long[_hub#_hardware]  'hardware >> 29
    $0001_0001 : PrintString(string("RamBlade"))
    $0001_0002 : PrintString(string("TriBlade#2"))
    $0001_0003 : PrintString(string("RamBlade3"))
    other: PrintString(string("???"))
  crlf

PRI StopAll | t

  waitcnt(clkfreq * 5 + cnt)
  repeat t from 0 to 7                                  'Stop all COGs except this one
    if (t <> cogid)
      cogstop(t)
  cogstop(cogid)                                        'then stop this cog. All done!

PRI HardwareFail | pu
  PrintString(string("** Cannot determine hardware ** $"))
  sio.hex(0,pu,8)
  crlf
  StopAll


PUB PrintStringCR(pstring)      ' sends string to output devices (vga/serial) with CRLF
    PrintString(pstring)                            
    crlf

PUB PrintString(pstring)        ' sends string to output devices (vga/serial) with no CRLF
    repeat strsize(pstring)
      PrintChar(byte[pstring++])
     
PUB PrintChar(c)                ' sends character to output devices (vga/serial)
    sio.tx(0,c)                                         ' send to com port
{
    if (_VGA_Enabled)
      vt100.out(c)                                      ' send to vga
    if (_Serial_Enabled)
      sio.tx(0,c)                                       ' send to com port
    if (_lcd_Enabled)
      LCD.out(c)                                        ' send to LCD display        
}

PUB CRLF
    PrintChar(_carriageReturnCharacter)                 ' send to vga
    if long[_hub#_toggleLF]
      PrintChar(_lineFeedCharacter)                     ' send to vga


{{

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                  TERMS OF USE: MIT License
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,
// modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all copies or substantial portions of the
// Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
// COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
}}