{{
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// General Purpose OS Command module _xxxxxxx.CMD called by _CMD.CMD, returns to _CMD.CMD 
//
// Derived from SD3.01 FATEngine Demo
// Author:        1/2/2011      Kwabena W. Agyeman     
// Modifications: J. Moxham Dr_Acula to create KyeDOS v3.
// Modifications: Cluso99 (Ray Rodrick) break into modules
// Copyright:     (c) 2011 Kwabena W. Agyeman  (Kye)
//                (c) 2011 J. Moxham           (Dr_Acula)
//                (c) 2012 Ray Rodrick         (Cluso99)
// Licence:       MIT (see end of file for details)
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Modifications by Cluso99...
// RR20120306  XXX_014 make general purpose base module
//                
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
}}

CON
  _clkmode = _hw#_clkmode
  _xinfreq = _hw#_xinfreq

  _nullCharacter                = 0
  _homeCursorCharacter          = 1
  _backspaceCharacter           = 8
  _horizontalTabCharacter       = 9
  _lineFeedCharacter            = 10
  _carriageReturnCharacter      = 13
  _clearScreenCharacter         = 16
  _quoteCharacter               = 34
  _backspaceKeycode             = 200

OBJ
  _hw   : "_hwdef"              'hardware pin definitions etc
  _hub  : "_hubdef"             'fixed hub definitions ($7FFF... down)

  fat   : "SD-MMC_FATEngine_001.spin"
  taf   : "SD-MMC_FATEngine_001.spin"
  str   : "ASCII0_STREngine.spin"
  sio   : "pcFullDuplexSerial2FC" 

VAR

   byte CommandLine[80] ' complete command line as typed in
   byte LineOfText1[80] ' general purpose string buffer
   byte LineOfText2[80] ' general purpose string buffer
   byte LineOfText3[80] ' general purpose string buffer

   
PUB shell | z,z0,z1,z2,z3,z4

  waitcnt(clkfreq/1000 * 500 + cnt)                     ' delay 500ms

{{
'==================
'force parameters in hub from module _CMD.CMD
  long[_hub#_SDPins]  :=  23 << 24 | 22 << 16 | 21 << 8 | 20  '(RamBlade3)                   
  long[_hub#_SIOPins] := 0 << 16 | 30 << 8 | 31
  long[_hub#_SIOBaud] := 115200                    
  long[_hub#_toggleLF] := false
  str.copy(string("???"),_hub#_stringBuffer)                 
'==================
}}

'get the hub parameters...
  z  :=  long[_hub#_sioPins]
  z0 :=  z        & $1F
  z1 := (z >>  8) & $1F
  z2 := (z >> 16) & $1F
' z3 := (z >> 24) & $1F
  z4 :=  long[_hub#_sioBaud]
  sio.AddPort(0, z0, z1, -1, -1, 0, z2, z4)
  sio.start                                             ' start the com port

  z  :=  long[_hub#_sdPins]
  z0 :=  z        & $1F
  z1 := (z >>  8) & $1F
  z2 := (z >> 16) & $1F
  z3 := (z >> 24) & $1F
'                          (DO, CLK, DI, CS, _SD_WP, _SD_CD, _RTC_DAT, _RTC_CLK, -1)
  ifnot( fat.FATEngineStart(z0, z1,  z2, z3, -1,     -1,     -1,       -1,       -1))
    reboot

  fat.mountPartition(0)                                 ' mount the sd card
'  crlf
  PrintStringCR(string("=== UNMOUNT ==="))              ' program message

  ifnot(fat.partitionMounted)
    PrintString(string("==Card Not Mounted=="))

  sio.rxflush(0)                                      

'=================
' execute the command
' str.copy(string("???"),_hub#_stringBuffer)                 
  str.copy(_hub#_stringBuffer,@commandline)             ' retrieve commandline 
  str.copy(@commandline,result)
  result := str.tokenizeString(result)
  result := \executeCommand(result)
'=================

' return to the command module
  \fat.bootPartition(string("_CMD.CMD"))
  'only here on error
  PrintStringCR(string("=== error ==="))
  waitcnt(clkfreq/1000 * 500 + cnt)                     ' delay 500ms
  reboot

{/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////}

PUB PrintHexadecimal(number, length)
    PrintString(str.integerToHexadecimal(number, length))

PUB PrintDecimal(number)        ' print the decimal value, useful for debugging
    PrintString(str.integerToDecimal(number, 10))    

PUB PrintStringCR(pstring)      ' sends string to output devices (vga/serial) with CRLF
    PrintString(pstring)                            
    crlf

PUB PrintString(pstring)        ' sends string to output devices (vga/serial) without CRLF
    repeat strsize(pstring)
      PrintChar(byte[pstring++])
     
PUB PrintChar(c)                ' sends character to output devices (vga/serial)
    sio.tx(0,c)                                         ' send to com port

PUB CRLF
    PrintChar(_carriageReturnCharacter)                 ' sends <cr> to output devices
    if long[_hub#_toggleLF]
      PrintChar(_lineFeedCharacter)                     ' sends <lf> to output devices

{/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////}
 
PRI executeCommand(stringPointer)

  ifnot(str.stringCompareCI(string("unmount"), stringPointer))
    taf.unmountPartition
    fat.unmountPartition
    printstringcr(string("Disk ready for removal"))

    abort 0

{/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////}

{{

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                  TERMS OF USE: MIT License
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,
// modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all copies or substantial portions of the
// Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
// COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
}}